"""本模块提供并行执行器, 及方便函数 `io_bound`, `cpu_bound`.
"""
import asyncio
import functools
import importlib
import multiprocessing
from asyncio.events import AbstractEventLoop
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
from typing import Any, Awaitable, Callable, ClassVar, Dict, Optional, Tuple

from ..typing import P, R


def IS_MAIN_PROCESS() -> bool:
    """返回是否为主进程

    Returns:
        bool: 是否为主进程
    """
    return multiprocessing.parent_process() is None


def _reg_sigint():
    """在子进程注册 SIGINT 处理函数以防止 Traceback 炸屏"""
    import signal
    import sys

    signal.signal(signal.SIGINT, lambda *_, **__: sys.exit())


class ParallelExecutor:
    """并行执行器."""

    thread_exec: ThreadPoolExecutor
    proc_exec: ProcessPoolExecutor
    loop_ref_dict: ClassVar[Dict[AbstractEventLoop, "ParallelExecutor"]] = {}
    func_mapping: ClassVar[Dict[Tuple[str, str], Callable]] = {}

    def __init__(
        self,
        loop: Optional[AbstractEventLoop] = None,
        max_thread: Optional[int] = None,
        max_process: Optional[int] = None,
    ):
        """初始化并行执行器.

        Args:
            loop (AbstractEventLoop, optional): 要绑定的事件循环, 会自动获取当前事件循环. Defaults to None.
            max_thread (int, optional): 最大线程数. Defaults to None.
            max_process (int, optional): 最大进程数. Defaults to None.

        `max_thread` 与 `max_process` 参数默认值请参阅 `concurrent.futures`.
        """
        self.thread_exec = ThreadPoolExecutor(max_workers=max_thread)
        self.proc_exec = ProcessPoolExecutor(
            max_workers=max_process, initializer=_reg_sigint
        )  # see issue #50
        self.bind_loop(loop or asyncio.get_running_loop())

    @classmethod
    def get(cls, loop: Optional[AbstractEventLoop] = None) -> "ParallelExecutor":
        """获取 ParallelExecutor 实例

        Args:
            loop (AbstractEventLoop, optional): 查找的事件循环. Defaults to None.

        Returns:
            ParallelExecutor: 找到的 / 新创建的 ParallelExecutor.
        """
        loop = loop or asyncio.get_running_loop()
        if loop not in cls.loop_ref_dict:
            cls.loop_ref_dict[loop] = ParallelExecutor()
        return cls.loop_ref_dict[loop]

    def bind_loop(self, loop: AbstractEventLoop):
        """绑定本实例到 loop.

        Args:
            loop (AbstractEventLoop): 要绑定到的事件循环.
        """
        self.loop_ref_dict[loop] = self

    @classmethod
    def shutdown(cls):
        """关闭本类的所有底层 Executor."""
        for exec in cls.loop_ref_dict.values():
            exec.close()

    def close(self):
        """关闭实例的所有底层 Executor."""
        self.thread_exec.shutdown()
        self.proc_exec.shutdown()

    @classmethod
    def run_func(cls, name: str, module: str, args: tuple, kwargs: dict) -> Any:
        """运行函数的实现

        Args:
            name (str): 函数名 (__qualname__)
            module (str): 函数所在模块名 (__module__)
            args (tuple): 位置参数
            kwargs (dict): 关键字参数

        Returns:
            Any: 底层函数的返回值
        """
        importlib.import_module(module)
        return cls.func_mapping[module, name](*args, **kwargs)

    @classmethod
    def run_func_static(cls, func: Callable[..., R], args: tuple, kwargs: dict) -> R:
        """调用一个静态函数 (会自动解包装已被 ParallelExecutor 包装过的函数)

        Args:
            func (Callable[..., R]): 要调用的函数
            args (tuple): 位置参数
            kwargs (dict): 关键字参数

        Returns:
            R: 底层函数的返回值
        """
        if (func.__module__, func.__qualname__) in cls.func_mapping:
            func = cls.func_mapping[func.__module__, func.__qualname__]
        return func(*args, **kwargs)  # type: ignore

    def to_thread(self, func: Callable[P, R], *args: P.args, **kwargs: P.kwargs) -> Awaitable[R]:
        """在线程中异步运行 func 函数.

        Args:
            func (Callable[P, R]): 要调用的函数.
            *args (P.args): 附带的位置参数.
            **kwargs (P.kwargs): 附带的关键词参数.

        Returns:
            Future[R]: 返回结果. 需要被异步等待.
        """
        return asyncio.get_running_loop().run_in_executor(  # type: ignore
            self.thread_exec,
            ParallelExecutor.run_func_static,
            func,
            args,
            kwargs,
        )

    def to_process(self, func: Callable[P, R], *args: P.args, **kwargs: P.kwargs) -> Awaitable[R]:
        """在进程中异步运行 func 函数. 需要先注册过才行.

        Args:
            func (Callable[P, R]): 要调用的函数.
            *args (P.args): 附带的位置参数.
            **kwargs (P.kwargs): 附带的关键词参数.

        Returns:
            Future[R]: 返回结果. 需要被异步等待.
        """
        return asyncio.get_running_loop().run_in_executor(  # type: ignore
            self.proc_exec,
            ParallelExecutor.run_func_static,
            func,
            args,
            kwargs,
        )


def io_bound(func: Callable[P, R]) -> Callable[P, Awaitable[R]]:
    """包装一个函数在线程中异步运行.

    Args:
        func (Callable[P, R]): 要包装的函数

    Returns:
        Callable[P, Awaitable[R]]: 包装后的函数
    """
    ParallelExecutor.func_mapping[func.__module__, func.__qualname__] = func

    @functools.wraps(func)
    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        executor = ParallelExecutor.get(loop)
        return await loop.run_in_executor(
            executor.thread_exec,
            ParallelExecutor.run_func,
            func.__qualname__,
            func.__module__,
            args,
            kwargs,
        )

    return wrapper


def cpu_bound(func: Callable[P, R]) -> Callable[P, Awaitable[R]]:
    """包装一个函数在进程中异步运行.

    Args:
        func (Callable[P, R]): 要包装的函数

    Returns:
        Callable[P, Awaitable[R]]: 包装后的函数
    """
    ParallelExecutor.func_mapping[func.__module__, func.__qualname__] = func

    @functools.wraps(func)
    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        executor = ParallelExecutor.get(loop)
        return await loop.run_in_executor(
            executor.proc_exec,
            ParallelExecutor.run_func,
            func.__qualname__,
            func.__module__,
            args,
            kwargs,
        )

    return wrapper



"""基于 format string 的消息链格式化器"""
import re
from typing import Dict, List, Union

from .chain import MessageChain
from .element import Element, Plain


class Formatter:
    """类似于 string.Formatter 的消息链格式化器"""

    format_string: str

    def __init__(self, format_string: str) -> None:
        self.format_string = format_string

    def format(
        self, *o_args: Union[Element, MessageChain, str], **o_kwargs: Union[Element, MessageChain, str]
    ) -> MessageChain:
        """通过初始化时传入的格式字符串 格式化消息链

        Args:
            *o_args (Union[Element, MessageChain, str]): 格式化时传入的关键字参数
            **o_kwargs (Union[Element, MessageChain, str]): 格式化时传入的关键字参数

        Returns:
            MessageChain: 格式化后的消息链
        """
        args: List[MessageChain] = [MessageChain.create(e) for e in o_args]
        kwargs: Dict[str, MessageChain] = {k: MessageChain.create(e) for k, e in o_kwargs.items()}

        args_mapping: Dict[str, MessageChain] = {
            f"\x02{index}\x02": chain for index, chain in enumerate(args)
        }
        kwargs_mapping: Dict[str, MessageChain] = {f"\x03{key}\x03": chain for key, chain in kwargs.items()}

        result = self.format_string.format(*args_mapping, **{k: f"\x03{k}\x03" for k in kwargs})

        chain_list: List[Union[MessageChain, Plain]] = []

        for i in re.split("([\x02\x03][\\d\\w]+[\x02\x03])", result):
            if match := re.fullmatch("(?P<header>[\x02\x03])(?P<content>\\w+)(?P=header)", i):
                header = match.group("header")
                full: str = match.group(0)
                if header == "\x02":  # from args
                    chain_list.append(args_mapping[full])
                else:  # \x03, from kwargs
                    chain_list.append(kwargs_mapping[full])
            else:
                chain_list.append(Plain(i))
        return MessageChain.create(*chain_list).merge()



"""Ariadne 消息事件"""
from typing import Union

from graia.broadcast.interfaces.dispatcher import DispatcherInterface

from ..dispatcher import (
    BaseDispatcher,
    MessageChainDispatcher,
    SenderDispatcher,
    SourceDispatcher,
    SubjectDispatcher,
)
from ..message.chain import MessageChain
from ..model import Client, Friend, Group, Member, Stranger
from ..typing import generic_issubclass
from . import MiraiEvent
from .mirai import FriendEvent, GroupEvent


class MessageEvent(MiraiEvent):
    """Ariadne 消息事件基类"""

    type: str = "MessageEvent"
    messageChain: MessageChain
    """消息链"""

    sender: Union[Friend, Member, Client, Stranger]
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]


class FriendMessage(MessageEvent, FriendEvent):
    """好友消息"""

    type: str = "FriendMessage"

    messageChain: MessageChain
    """消息链"""

    sender: Friend
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]


class GroupMessage(MessageEvent, GroupEvent):
    """群组消息"""

    type: str = "GroupMessage"

    messageChain: MessageChain
    """消息链"""

    sender: Member
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]

        @staticmethod
        async def catch(interface: DispatcherInterface):
            if isinstance(interface.event, GroupMessage):
                if generic_issubclass(Group, interface.annotation):
                    return interface.event.sender.group


class TempMessage(MessageEvent):
    """临时消息"""

    type: str = "TempMessage"

    messageChain: MessageChain
    """消息链"""

    sender: Member
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]

        @staticmethod
        async def catch(interface: DispatcherInterface):
            if isinstance(interface.event, TempMessage):
                if generic_issubclass(Group, interface.annotation):
                    return interface.event.sender.group


class OtherClientMessage(MessageEvent):
    """其他客户端消息"""

    type: str = "OtherClientMessage"

    messageChain: MessageChain
    """消息链"""

    sender: Client
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]


class StrangerMessage(MessageEvent):
    """陌生人消息"""

    type: str = "StrangerMessage"

    messageChain: MessageChain
    """消息链"""

    sender: Stranger
    """发送者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SenderDispatcher]


class ActiveMessage(MiraiEvent):
    """主动消息: Bot 账号发送给他人的消息"""

    type: str

    messageChain: MessageChain
    """消息链"""

    subject: Union[Friend, Group, Member, Stranger]
    """消息接收者"""

    sync: bool = False
    """是否为同步消息"""


class ActiveFriendMessage(ActiveMessage):
    """主动好友消息"""

    type: str = "ActiveFriendMessage"

    messageChain: MessageChain
    """消息链"""

    subject: Friend
    """消息接收者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SubjectDispatcher]


class ActiveGroupMessage(ActiveMessage):
    """主动群组消息"""

    type: str = "ActiveGroupMessage"

    messageChain: MessageChain
    """消息链"""

    subject: Group
    """消息接收者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SubjectDispatcher]


class ActiveTempMessage(ActiveMessage):
    """主动临时消息"""

    type: str = "ActiveTempMessage"

    messageChain: MessageChain
    """消息链"""

    subject: Member
    """消息接收者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SubjectDispatcher]

        @staticmethod
        async def catch(interface: DispatcherInterface):
            if isinstance(interface.event, ActiveTempMessage):
                if interface.annotation is Group:
                    return interface.event.subject.group


class ActiveStrangerMessage(ActiveMessage):
    """主动陌生人消息"""

    type: str = "ActiveStrangerMessage"

    messageChain: MessageChain
    """消息链"""

    subject: Stranger
    """消息接收者"""

    class Dispatcher(BaseDispatcher):
        mixin = [MessageChainDispatcher, SourceDispatcher, SubjectDispatcher]


class FriendSyncMessage(ActiveFriendMessage):
    """好友同步消息"""

    type: str = "FriendSyncMessage"

    sync = True


class GroupSyncMessage(ActiveGroupMessage):
    """群组同步消息"""

    type: str = "GroupSyncMessage"

    sync = True


class TempSyncMessage(ActiveTempMessage):
    """临时会话同步消息"""

    type: str = "TempSyncMessage"

    sync = True


class StrangerSyncMessage(ActiveStrangerMessage):
    """陌生人同步消息"""

    type: str = "StrangerSyncMessage"

    sync = True



"""Ariadne 一站式导入的提供模块"""


import contextlib

from ..adapter import ComposeForwardAdapter as ComposeForwardAdapter
from ..adapter import DefaultAdapter as DefaultAdapter
from ..adapter import HttpAdapter as HttpAdapter
from ..adapter import WebsocketAdapter as WebsocketAdapter
from ..app import Ariadne as Ariadne
from ..context import adapter_ctx as adapter_ctx
from ..context import ariadne_ctx as ariadne_ctx
from ..context import broadcast_ctx as broadcast_ctx
from ..context import event_ctx as event_ctx
from ..context import event_loop_ctx as event_loop_ctx
from ..context import upload_method_ctx as upload_method_ctx
from ..dispatcher import ContextDispatcher as ContextDispatcher
from ..dispatcher import MessageChainDispatcher as MessageChainDispatcher
from ..dispatcher import SourceDispatcher as SourceDispatcher
from ..exception import AccountMuted as AccountMuted
from ..exception import AccountNotFound as AccountNotFound
from ..exception import InvalidArgument as InvalidArgument
from ..exception import InvalidEventTypeDefinition as InvalidEventTypeDefinition
from ..exception import InvalidSession as InvalidSession
from ..exception import InvalidVerifyKey as InvalidVerifyKey
from ..exception import MessageTooLong as MessageTooLong
from ..exception import UnknownError as UnknownError
from ..exception import UnknownTarget as UnknownTarget
from ..exception import UnVerifiedSession as UnVerifiedSession
from ..model import BotMessage as BotMessage
from ..model import CallMethod as CallMethod
from ..model import ChatLogConfig as ChatLogConfig
from ..model import Client as Client
from ..model import DownloadInfo as DownloadInfo
from ..model import FileInfo as FileInfo
from ..model import Friend as Friend
from ..model import Group as Group
from ..model import GroupConfig as GroupConfig
from ..model import Member as Member
from ..model import MemberInfo as MemberInfo
from ..model import MemberPerm as MemberPerm
from ..model import MiraiSession as MiraiSession
from ..model import Profile as Profile
from ..model import Stranger as Stranger
from ..model import UploadMethod as UploadMethod
from ..util import cpu_bound as cpu_bound
from ..util import io_bound as io_bound
from ..util.cooldown import CoolDown as CoolDown
from ..util.send import Bypass as Bypass
from ..util.send import Ignore as Ignore
from ..util.send import Safe as Safe
from ..util.send import Strict as Strict
from . import event as event
from . import message as message

# no error


with contextlib.suppress(ImportError):
    from ..adapter.reverse import (
        ComposeReverseWebsocketAdapter as ComposeReverseWebsocketAdapter,
    )
    from ..adapter.reverse import ComposeWebhookAdapter as ComposeWebhookAdapter
    from ..adapter.reverse import ReverseWebsocketAdapter as ReverseWebsocketAdapter



from typing import Any, Dict, Type, Union

from ..exception import (
    AccountMuted,
    AccountNotFound,
    InvalidArgument,
    InvalidSession,
    InvalidVerifyKey,
    MessageTooLong,
    RemoteException,
    UnknownError,
    UnknownTarget,
    UnVerifiedSession,
)

code_exceptions_mapping: Dict[int, Type[Exception]] = {
    1: InvalidVerifyKey,
    2: AccountNotFound,
    3: InvalidSession,
    4: UnVerifiedSession,
    5: UnknownTarget,
    6: FileNotFoundError,
    10: PermissionError,
    20: AccountMuted,
    30: MessageTooLong,
    400: InvalidArgument,
    500: RemoteException,
}


def validate_response(data: Dict[str, Any]) -> Union[dict, Exception]:
    """验证远程服务器的返回值

    Args:
        data (dict): 返回的对象

    Raises:
        Exception: 请参照 code_exceptions_mapping
    """
    if isinstance(data, dict):
        int_code: int = data.get("code")
    else:
        int_code = data
    if not isinstance(int_code, int) or int_code == 200 or int_code == 0:
        if "data" in data:
            return data["data"]
        return data
    exc_cls = code_exceptions_mapping.get(int_code)
    if exc_cls:
        return exc_cls(exc_cls.__doc__, data)
    return UnknownError(data)



"""Ariadne 控制台对 Saya 封装的 Behaviour 与 Schema"""
from dataclasses import dataclass, field
from typing import Callable, List

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.saya.behaviour import Behaviour
from graia.saya.cube import Cube
from graia.saya.schema import BaseSchema

from . import Console


@dataclass
class ConsoleSchema(BaseSchema):
    """控制台监听 Schema, 相当于 console.register"""

    dispatchers: List[BaseDispatcher] = field(default_factory=list)
    decorators: List[Decorator] = field(default_factory=list)

    def register(self, func: Callable, console: Console):
        """注册 func 至 console

        Args:
            func (Callable): 监听函数
            console (Console): 注册到的 console
        """
        console.register(self.dispatchers, self.decorators)(func)


class ConsoleBehaviour(Behaviour):
    """控制台的 Saya Behaviour 实现, 传入 Console 对象"""

    def __init__(self, console: Console) -> None:
        self.console = console

    def allocate(self, cube: Cube[ConsoleSchema]):
        if not isinstance(cube.metaclass, ConsoleSchema):
            return
        cube.metaclass.register(cube.content, self.console)
        return True

    def uninstall(self, cube: Cube[ConsoleSchema]):
        if not isinstance(cube.metaclass, ConsoleSchema):
            return
        for val in self.console.registry[:]:
            if cube.content is val[0]:
                self.console.registry.remove(val)
        return True



from typing import Callable, Generic, List, Optional, Type, TypeVar, cast

from graia.broadcast import Broadcast
from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.event import Dispatchable
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interrupt import InterruptControl, Waiter
from graia.broadcast.typing import T_Dispatcher

from .. import get_running
from ..typing import T


class FunctionWaiter(Waiter, Generic[T]):
    """将 Waiter.create_using_function 封装了一层"""

    def __init__(
        self,
        func: Callable[..., T],
        events: List[Type[Dispatchable]],
        dispatchers: Optional[List[T_Dispatcher]] = None,
        decorators: Optional[List[Decorator]] = None,
        priority: int = 15,
    ) -> None:
        """
        Args:
            func (Callable): 调用函数
            events (List[Type[Dispatchable]]): 事件类型
            dispatchers (Optional[List[T_Dispatcher]]): 广播器
            decorators (Optional[List[Decorator]]): 装饰器
            priority (int): 优先级
        """
        self.listening_events = self.events = events
        self.using_dispatchers = self.dispatchers = dispatchers or []
        self.using_decorators = self.decorators = decorators or []
        self.priority = priority
        self.detected_event = func  # type: ignore

    async def wait(
        self,
        timeout: Optional[float] = None,
    ) -> T:
        inc: InterruptControl = InterruptControl(get_running(Broadcast))
        return await inc.wait(
            self,
            timeout=timeout,  # type: ignore
        )


T_E = TypeVar("T_E", bound=Dispatchable)


class EventWaiter(Waiter, Generic[T_E]):
    """将 Waiter.create_using_event 封装了一层"""

    def __init__(
        self,
        events: List[Type[T_E]],
        dispatchers: Optional[List[T_Dispatcher]] = None,
        decorators: Optional[List[Decorator]] = None,
        extra_validator: Optional[Callable[[T_E], bool]] = None,
        priority: int = 15,
    ) -> None:
        """
        Args:
            events (List[Type[T_E]]): 事件类型
            dispatchers (Optional[List[T_Dispatcher]], optional): Dispatcher 列表
            decorators (Optional[List[Decorator]], optional): Decorator 列表
            extra_validator (Optional[Callable[[T_E], bool]], optional): 额外的验证器
            priority (int, optional): 优先级, 越小越靠前
        """
        self.events = events
        self.listening_events = cast(List[Type[Dispatchable]], self.events)
        self.using_dispatchers = self.dispatchers = dispatchers or []
        self.using_decorators = self.decorators = decorators or []
        self.extra_validator = extra_validator
        self.priority = priority

    async def detected_event(self, ev: Dispatchable) -> T_E:
        event = cast(T_E, ev)
        if self.extra_validator and not self.extra_validator(event):
            raise ExecutionStop
        return event

    async def wait(
        self,
        timeout: Optional[float] = None,
    ) -> T_E:
        inc: InterruptControl = InterruptControl(get_running(Broadcast))
        return await inc.wait(
            self,
            timeout=timeout,  # type: ignore
        )



"""Ariadne 基础的 parser, 包括 DetectPrefix 与 DetectSuffix"""
import abc
import difflib
import fnmatch
import re
import weakref
from typing import ClassVar, DefaultDict, Dict, List, Optional, Tuple, Type, Union

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.decorator import DecoratorInterface
from graia.broadcast.interfaces.dispatcher import DispatcherInterface
from loguru import logger

from ... import get_running
from ...event.message import GroupMessage
from ...typing import generic_issubclass
from ..chain import MessageChain
from ..element import At, Element, Plain, Quote, Source


class Compose(Decorator):
    """将多个基础 Decorator 串联起来"""

    def __init__(self, *deco: "ChainDecorator") -> None:
        self.deco: List[ChainDecorator] = list(deco)

    async def target(self, interface: DecoratorInterface):
        chain = await interface.dispatcher_interface.lookup_param("message_chain", MessageChain, None)
        for deco in self.deco:
            chain = await deco.decorate(chain, interface)
            if chain is None:
                break
        if interface.annotation is MessageChain:
            if chain is None:
                raise ExecutionStop
            return chain


class ChainDecorator(abc.ABC, Decorator):
    pre = True

    @abc.abstractmethod
    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        ...

    async def target(self, interface: DecoratorInterface):
        return await self.decorate(
            await interface.dispatcher_interface.lookup_param("message_chain", MessageChain, None), interface
        )


class DetectPrefix(ChainDecorator):
    """前缀检测器"""

    def __init__(self, prefix: str) -> None:
        """初始化前缀检测器.

        Args:
            prefix (str): 要匹配的前缀
        """
        self.prefix = prefix

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        if not rest.startswith(self.prefix):
            raise ExecutionStop
        result = rest.removeprefix(self.prefix).removeprefix(" ")
        if interface.annotation is MessageChain:
            return header + result


class DetectSuffix(ChainDecorator):
    """后缀检测器"""

    def __init__(self, suffix: str) -> None:
        """初始化后缀检测器.

        Args:
            suffix (str): 要匹配的后缀
        """
        self.suffix = suffix

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        if not rest.endswith(self.suffix):
            raise ExecutionStop
        result = rest.removesuffix(self.suffix).removesuffix(" ")
        if interface.annotation is MessageChain:
            return header + result


class MentionMe(ChainDecorator):
    """At 账号或者提到账号群昵称"""

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        ariadne = get_running()
        if isinstance(interface.event, GroupMessage):
            if not ariadne.account:
                logger.warning("Unable to detect Ariadne's name because account is not set")
                raise ExecutionStop
            name = (await ariadne.getMember(interface.event.sender.group, ariadne.account)).name
        else:
            name = (await ariadne.getBotProfile()).nickname
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        first: Element = rest[0]
        result: Optional[MessageChain] = None
        if rest and isinstance(first, Plain) and first.asDisplay().startswith(name):
            result = header + rest.removeprefix(name).removeprefix(" ")
        if rest and isinstance(first, At) and first.target == ariadne.account:
            result = header + MessageChain(rest.__root__[1:], inline=True).removeprefix(" ")

        if result is None:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return result


class Mention(ChainDecorator):
    """At 或提到指定人"""

    def __init__(self, target: Union[int, str]) -> None:
        self.person: Union[int, str] = target

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        header = chain.include(Quote, Source)
        rest: MessageChain = chain.exclude(Quote, Source)
        first: Element = rest[0]
        result: Optional[MessageChain] = None
        if (
            rest
            and isinstance(first, Plain)
            and isinstance(self.person, str)
            and first.asDisplay().startswith(self.person)
        ):
            result = header + rest.removeprefix(self.person).removeprefix(" ")
        if rest and isinstance(first, At) and isinstance(self.person, int) and first.target == self.person:
            result = header + MessageChain(rest.__root__[1:], inline=True).removeprefix(" ")

        if result is None:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return result


class ContainKeyword(ChainDecorator):
    """消息中含有指定关键字"""

    def __init__(self, keyword: str) -> None:
        self.keyword: str = keyword

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if self.keyword not in chain:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchContent(ChainDecorator):
    """匹配字符串 / 消息链"""

    def __init__(self, content: Union[str, MessageChain]) -> None:
        self.content: Union[str, MessageChain] = content
        self.next: Optional[ChainDecorator] = None

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if isinstance(self.content, str) and chain.asDisplay() != self.content:
            raise ExecutionStop
        if isinstance(self.content, MessageChain) and chain != self.content:
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchRegex(ChainDecorator):
    """匹配正则表达式"""

    def __init__(self, regex: str, flags: re.RegexFlag = re.RegexFlag(0)) -> None:
        """初始化匹配正则表达式.

        Args:
            regex (str): 正则表达式
            flags (re.RegexFlag): 正则表达式标志
        """
        self.regex: str = regex
        self.flags: re.RegexFlag = flags

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if not re.match(self.regex, chain.asDisplay(), self.flags):
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class MatchTemplate(ChainDecorator):
    """模板匹配"""

    def __init__(self, template: List[Union[Type[Element], Element]]) -> None:
        self.template: List[Union[Type[Element], Element, str]] = []
        for element in template:
            if isinstance(element, type) and element is not Plain:
                self.template.append(element)
            elif isinstance(element, Element) and not isinstance(element, Plain):
                self.template.append(element)
            else:
                element = element.text if isinstance(element, Plain) else "*"
                if self.template and isinstance(self.template[-1], str):
                    self.template[-1] = self.template[-1] + element
                else:
                    self.template.append(element)

    def match(self, chain: MessageChain):
        """匹配消息链"""
        chain = chain.asSendable()
        if len(self.template) != len(chain):
            return False
        for element, template in zip(chain, self.template):
            if isinstance(template, type) and not isinstance(element, template):
                return False
            elif isinstance(template, Element) and element != template:
                return False
            elif isinstance(template, str):
                if not isinstance(element, Plain) or not fnmatch.fnmatch(element.text, template):
                    return False
        return True

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if not self.match(chain):
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class FuzzyMatch(ChainDecorator):
    """模糊匹配

    Warning:
        我们更推荐使用 FuzzyDispatcher 来进行模糊匹配操作, 因为其具有上下文匹配数量限制.
    """

    def __init__(self, template: str, min_rate: float = 0.6) -> None:
        self.template: str = template
        self.min_rate: float = min_rate

    def match(self, chain: MessageChain):
        """匹配消息链"""
        text_frags: List[str] = []
        for element in chain:
            if isinstance(element, Plain):
                text_frags.append(element.text)
            else:
                text_frags.append(element.asDisplay())
        text = "".join(text_frags)
        matcher = difflib.SequenceMatcher(a=text, b=self.template)
        # return false when **any** ratio calc falls undef the rate
        if matcher.real_quick_ratio() < self.min_rate:
            return False
        if matcher.quick_ratio() < self.min_rate:
            return False
        return matcher.ratio() >= self.min_rate

    async def decorate(self, chain: MessageChain, interface: DecoratorInterface) -> Optional[MessageChain]:
        if not self.match(chain):
            raise ExecutionStop
        if interface.annotation is MessageChain:
            return chain


class FuzzyDispatcher(BaseDispatcher):
    scope_map: ClassVar[DefaultDict[str, List[str]]] = DefaultDict(list)
    event_ref: ClassVar["Dict[int, Dict[str, Tuple[str, float]]]"] = {}

    def __init__(self, template: str, min_rate: float = 0.6, scope: str = "") -> None:
        self.template: str = template
        self.min_rate: float = min_rate
        self.scope: str = scope
        self.scope_map[scope].append(template)

    async def beforeExecution(self, interface: DispatcherInterface):
        event = interface.event
        if id(event) not in self.event_ref:
            chain: MessageChain = await interface.lookup_param("message_chain", MessageChain, None)
            text_frags: List[str] = []
            for element in chain:
                if isinstance(element, Plain):
                    text_frags.append(element.text)
                else:
                    text_frags.append(element.asDisplay())
            text = "".join(text_frags)
            matcher = difflib.SequenceMatcher()
            matcher.set_seq2(text)
            rate_calc = self.event_ref[id(event)] = {}
            weakref.finalize(event, lambda d: self.event_ref.pop(d), id(event))
            for scope, templates in self.scope_map.items():
                max_match: float = 0.0
                for template in templates:
                    matcher.set_seq1(template)
                    if matcher.real_quick_ratio() < max_match:
                        continue
                    if matcher.quick_ratio() < max_match:
                        continue
                    if matcher.ratio() < max_match:
                        continue
                    rate_calc[scope] = (template, matcher.ratio())
                    max_match = matcher.ratio()
        win_template, win_rate = self.event_ref[id(event)].get(self.scope, (self.template, 0.0))
        if win_template != self.template or win_rate < self.min_rate:
            raise ExecutionStop

    async def catch(self, i: DispatcherInterface) -> Optional[float]:
        event = i.event
        _, rate = self.event_ref[id(event)].get(self.scope, (self.template, 0.0))
        if generic_issubclass(float, i.annotation) and "rate" in i.name:
            return rate



"""Ariadne 的适配器"""
import abc
import asyncio
import contextlib
from asyncio import Queue, Task
from typing import Any, Dict, FrozenSet, Optional, Union

from aiohttp import ClientSession, FormData
from graia.broadcast import Broadcast
from loguru import logger

from ..event import MiraiEvent
from ..exception import InvalidArgument
from ..model import CallMethod, MiraiSession
from ..util import AsyncSignal


class Adapter(abc.ABC):

    tags: FrozenSet[str] = frozenset(["abstract"])

    def __init__(self, broadcast: Broadcast, mirai_session: MiraiSession) -> None:
        """初始化 Adapter.

        Args:
            broadcast (Broadcast): 广播系统
            mirai_session (MiraiSession): MiraiSession 实例
        """
        self.broadcast: Broadcast = broadcast
        self.running: bool = False
        self.mirai_session: MiraiSession = mirai_session
        self.fetch_task: Optional[Task] = None
        self.call_task: Optional[Task] = None
        self.session: Optional[ClientSession] = None
        self.event_queue: Optional[Queue[MiraiEvent]] = None
        self.connected: AsyncSignal[bool] = AsyncSignal(False)
        if "abstract" in self.tags:
            raise TypeError("Adapter is abstract, cannot be instantiated.")

    @abc.abstractmethod
    async def call_api(
        self,
        action: str,
        method: CallMethod,
        data: Optional[Union[Dict[str, Any], str, FormData]] = None,
    ) -> Dict[Any, Any]:
        """调用 API

        Args:
            action (str): API 动作名, 用斜杠分割
            method (CallMethod): 调用方法
            data (Optional[Union[Dict[str, Any], str, FormData]], optional): 调用数据. Defaults to None.

        Returns:
            dict: API 返回的数据, 为 json 兼容格式
        """
        ...

    async def start(self):
        """启动 Adapter"""
        if not self.running:
            self.running = True
            self.event_queue = Queue()
            self.fetch_task = asyncio.create_task(self.fetch_cycle())
            if self.mirai_session.single_mode:
                self.connected.set(True)
        if "reverse" not in self.tags:
            await self.connected.wait(True)

    def build_event(self, data: dict) -> MiraiEvent:
        """
        从尚未明确指定事件类型的对象中获取事件的定义, 并进行解析

        Args:
            data (dict): 用 dict 表示的序列化态事件, 应包含有字段 `type` 以供分析事件定义.

        Raises:
            InvalidArgument: 目标对象中不包含字段 `type`
            ValueError: 没有找到对应的字段, 通常的, 这意味着应用获取到了一个尚未被定义的事件, 请报告问题.

        Returns:
            MiraiEvent: 已经被序列化的事件
        """
        event_type: Optional[str] = data.get("type")
        if not event_type or not isinstance(event_type, str):
            raise InvalidArgument("Unable to find 'type' field for automatic parsing", data)
        event_class: Optional[MiraiEvent] = self.broadcast.findEvent(event_type)  # type: ignore
        if not event_class:
            logger.error("An event is not recognized! Please report with your log to help us diagnose.")
            raise ValueError(f"Unable to find event: {event_type}", data)
        data = {k: v for k, v in data.items() if k != "type"}
        return event_class.parse_obj(data)

    @abc.abstractmethod
    async def fetch_cycle(self):
        """循环获取事件, 放入 event_queue 中"""
        ...

    async def stop(self):
        """停止 Adapter"""
        self.running = False
        if self.fetch_task:
            await self.fetch_task
        logger.success("Event fetch task completed.")
        self.fetch_task = None


from .forward import ComposeForwardAdapter as ComposeForwardAdapter  # noqa: F401, E402
from .forward import HttpAdapter as HttpAdapter  # noqa: F401, E402
from .forward import WebsocketAdapter as WebsocketAdapter  # noqa: F401, E402

DefaultAdapter = ComposeForwardAdapter


class DebugAdapter(DefaultAdapter):
    """调试 Adapter"""

    def build_event(self, data: dict) -> MiraiEvent:
        event = super().build_event(data)
        logger.debug(event)
        return event


with contextlib.suppress(ImportError):
    from .reverse import (  # noqa: F401, E402
        ComposeReverseWebsocketAdapter as ComposeReverseWebsocketAdapter,
    )
    from .reverse import (  # noqa: F401, E402
        ComposeWebhookAdapter as ComposeWebhookAdapter,
    )
    from .reverse import ReverseAdapter as ReverseAdapter  # noqa: F401, E402
    from .reverse import (  # noqa: F401, E402
        ReverseWebsocketAdapter as ReverseWebsocketAdapter,
    )



"""Ariadne 的事件"""
from graia.broadcast import Dispatchable
from pydantic import validator

from ..dispatcher import BaseDispatcher
from ..exception import InvalidEventTypeDefinition
from ..model import AriadneBaseModel


class MiraiEvent(Dispatchable, AriadneBaseModel):
    """Ariadne 的事件基类"""

    type: str
    """事件类型"""

    @validator("type", allow_reuse=True)
    def validate_event_type(cls, v):
        """验证事件类型, 通过比对 type 字段实现"""
        if not isinstance(cls, type):
            raise TypeError("cls must be a class!")
        if cls.type != v:
            raise InvalidEventTypeDefinition(f"{cls.__name__}'s type must be '{cls.type}', not '{v}'")
        return v

    Dispatcher = BaseDispatcher



"""正向 Adapter, 作为客户端连接至 mirai-api-http"""

import asyncio
import json
from asyncio import CancelledError, Task
from typing import Any, Dict, FrozenSet, List, Optional, Tuple, Union

from aiohttp import (
    ClientConnectionError,
    ClientSession,
    ClientWebSocketResponse,
    FormData,
    WebSocketError,
    WSMsgType,
)
from graia.broadcast import Broadcast
from loguru import logger
from yarl import URL

from ..exception import InvalidSession
from ..model import CallMethod, DatetimeEncoder, MiraiSession
from ..util import yield_with_timeout
from . import Adapter
from .util import validate_response


class HttpAdapter(Adapter):
    """
    仅使用正向 HTTP 的适配器, 采用短轮询接收事件/消息.
    """

    tags: FrozenSet[str] = frozenset(["http", "forward"])

    def __init__(
        self,
        broadcast: Broadcast,
        mirai_session: MiraiSession,
        fetch_interval: float = 0.5,
        count: int = 10,
    ) -> None:
        """初始化 HttpAdapter

        Args:
            broadcast (Broadcast): 广播系统
            mirai_session (MiraiSession): MiraiSession 实例
            fetch_interval (float, optional): 抓取事件间隔(s). Defaults to 0.5.
            count (int, optional): 单次抓取事件数目. Defaults to 10.
        """
        super().__init__(broadcast, mirai_session)
        self.fetch_interval = fetch_interval
        self.count = count

    async def authenticate(self) -> None:
        """在 mira-api-http 进行认证并存入 MiraiSession"""
        if not self.mirai_session.session_key:
            async with self.session.post(
                self.mirai_session.url_gen("verify"),
                data=json.dumps({"verifyKey": self.mirai_session.verify_key}),
            ) as response:
                response.raise_for_status()
                session_key: dict = (await response.json())["session"]
            async with self.session.post(
                self.mirai_session.url_gen("bind"),
                data=json.dumps({"sessionKey": session_key, "qq": self.mirai_session.account}),
            ) as response:
                response.raise_for_status()
                validate_response(await response.json())
            self.mirai_session.session_key = session_key
            logger.success("Successfully got session key")
            self.connected.set(True)

    async def fetch_cycle(self) -> None:
        async with ClientSession() as session:
            self.session = session
            await self.authenticate()
            while self.running:
                await asyncio.sleep(self.fetch_interval)
                async with self.session.get(
                    URL(self.mirai_session.url_gen("fetchMessage")).with_query(
                        {"sessionKey": self.mirai_session.session_key, "count": self.count}
                    )
                ) as response:
                    response.raise_for_status()
                    resp_json: dict = await response.json()
                    resp: List[dict] = validate_response(resp_json)
                for data in resp:
                    await self.event_queue.put(self.build_event(data))
            self.connected.set(False)
            self.mirai_session.session_key = None

    async def call_api(
        self,
        action: str,
        method: CallMethod,
        data: Optional[Union[Dict[str, Any], str, FormData]] = None,
    ) -> Union[dict, list]:
        await self.connected.wait(True)
        if method in (CallMethod.GET, CallMethod.RESTGET):
            if isinstance(data, str):
                data = json.loads(data)
            async with self.session.get(URL(self.mirai_session.url_gen(action)).with_query(data)) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()

        elif method in (CallMethod.POST, CallMethod.RESTPOST):
            if not isinstance(data, str):
                data = json.dumps(data, cls=DatetimeEncoder)
            async with self.session.post(self.mirai_session.url_gen(action), data=data) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()

        else:  # MULTIPART
            if isinstance(data, FormData):
                form = data
            elif isinstance(data, dict):
                form = FormData(quote_fields=False)
                for k, v in data.items():
                    v: Union[str, bytes, Tuple[Any, dict]]
                    if isinstance(v, tuple):
                        form.add_field(k, v[0], **v[1])
                    else:
                        form.add_field(k, v)
            async with self.session.post(self.mirai_session.url_gen(action), data=form) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()

        val = validate_response(resp_json)
        if not isinstance(val, Exception):
            return val
        if isinstance(val, InvalidSession):
            self.connected.set(False)
            self.mirai_session.session_key = None
        raise val


class WebsocketAdapter(Adapter):
    """
    正向 Websocket 适配器.
    """

    tags: FrozenSet[str] = frozenset(["websocket", "forward"])

    def __init__(
        self, broadcast: Broadcast, mirai_session: MiraiSession, ping: bool = True, log: bool = False
    ) -> None:
        """初始化 WebsocketAdapter

        Args:
            broadcast (Broadcast): 广播系统
            mirai_session (MiraiSession): MiraiSession 实例
            ping (bool, optional): 是否发送 ping 消息. Defaults to True.
            log (bool, optional): 是否记录网络日志. Defaults to False.
        """
        super().__init__(broadcast, mirai_session)
        self.ping = ping
        self.ping_task: Optional[Task] = None
        self.websocket: Optional[ClientWebSocketResponse] = None
        self.query_dict = {"verifyKey": mirai_session.verify_key}
        self.future_map: Dict[str, asyncio.Future] = {}
        self.log = log
        if not mirai_session.single_mode:
            self.query_dict["qq"] = mirai_session.account

    async def ws_ping(self, interval: float = 30.0) -> None:
        """向 Mirai API HTTP 的 WebsocketAdapter 循环发送 ping.

        Args:
            interval (float, optional): ping 间隔 (s). 默认 30.0.
        """
        while self.running:
            try:
                try:
                    await self.websocket.ping()
                    if self.log:
                        logger.debug("websocket: ping")
                except Exception as e:
                    logger.exception(f"websocket: ping failed: {e!r}")
                else:
                    if self.log:
                        logger.debug(f"websocket: ping success, delay {interval}s")
                    await asyncio.sleep(interval)
            except asyncio.CancelledError:
                if self.log:
                    logger.debug("websocket: pinger exit")
                break

    async def call_api(
        self, action: str, method: CallMethod, data: Optional[Union[Dict[str, Any], str, FormData]] = None
    ) -> Union[dict, list]:
        await self.connected.wait(True)
        future = self.broadcast.loop.create_future()
        self.future_map[str(id(future))] = future
        content = {
            "syncId": str(id(future)),
            "command": action.replace("/", "_"),
            "content": data,
        }
        if method == CallMethod.RESTGET:
            content["subCommand"] = "get"
        elif method == CallMethod.RESTPOST:
            content["subCommand"] = "update"
        elif method == CallMethod.MULTIPART:
            future.set_exception(
                NotImplementedError(f"Unsupported operation for ReverseWebsocketAdapter: {method}")
            ),
        await self.websocket.send_str(json.dumps(content, cls=DatetimeEncoder))
        return await future

    async def fetch_cycle(self) -> None:
        self.running = True
        async with ClientSession() as session:
            self.session = session
            try:
                async with self.session.ws_connect(
                    str(URL(self.mirai_session.url_gen("all")).with_query(self.query_dict)),
                    autoping=False,
                ) as connection:
                    logger.info("websocket: connected")
                    self.websocket = connection

                    if self.ping:
                        self.ping_task = self.broadcast.loop.create_task(
                            self.ws_ping(), name="ariadne_adapter_ws_ping"
                        )
                        logger.info("websocket: ping task created")

                    async for ws_message in yield_with_timeout(connection.receive, lambda: self.running):
                        if ws_message.type is WSMsgType.TEXT:
                            raw_data: dict = json.loads(ws_message.data)
                            sync_id: int = int(raw_data["syncId"] or -1)
                            data: dict = raw_data["data"]
                            if "session" in data:
                                self.mirai_session.session_key = data["session"]
                                logger.success("Successfully got session key")
                                self.connected.set(True)
                                continue
                            if sync_id in self.future_map:
                                fut = self.future_map.pop(str(sync_id))
                                res = validate_response(data)
                                if isinstance(res, Exception):
                                    fut.set_exception(res)
                                else:
                                    fut.set_result(res)
                            else:
                                await self.event_queue.put(self.build_event(data))
                        elif ws_message.type is WSMsgType.CLOSED:
                            logger.warning("websocket: connection has been closed.")
                            raise WebSocketError(1, "connection closed")
                        elif ws_message.type is WSMsgType.PONG:
                            if self.log:
                                logger.debug("websocket: received pong")
                        else:
                            logger.warning(f"websocket: unknown message type - {ws_message.type}")
            except CancelledError:
                pass
            except ClientConnectionError as e:
                logger.error(f"{e.__class__.__name__}: {e}")
            except Exception as e:
                logger.exception(e)
            finally:
                if self.ping_task:
                    self.ping_task.cancel()
                    self.ping_task = None
                    if self.log:
                        logger.debug("websocket: ping task complete")
                if self.websocket:
                    logger.info("websocket: disconnected")
                self.running = False
                self.connected.set(False)
                self.mirai_session.session_key = None
                self.websocket = None
                self.session = None


class ComposeForwardAdapter(WebsocketAdapter):
    """正向 HTTP 与 Websocket 的组合适配器."""

    tags: FrozenSet[str] = frozenset(["websocket", "http", "forward"])

    call_api = HttpAdapter.call_api



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['graia',
 'graia.ariadne',
 'graia.ariadne.adapter',
 'graia.ariadne.console',
 'graia.ariadne.entry',
 'graia.ariadne.event',
 'graia.ariadne.message',
 'graia.ariadne.message.commander',
 'graia.ariadne.message.parser',
 'graia.ariadne.util']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0',
 'graia-broadcast>=0.16.1,<0.17.0',
 'loguru>=0.6,<0.7',
 'pydantic>=1.8.2,<2.0.0',
 'typing-extensions>=4.0,<5.0',
 'yarl>=1.7,<2.0']

extras_require = \
{'alconna': ['arclet-alconna-graia>=0.0,<0.1'],
 'full': ['arclet-alconna-graia>=0.0,<0.1',
          'graia-scheduler>=0.0,<0.1',
          'graia-saya>=0.0,<0.1',
          'fastapi>=0.74.1,<1.0.0',
          'uvicorn[standard]>=0.17.5,<0.18.0',
          'prompt-toolkit>=3.0.24,<4.0.0'],
 'graia': ['graia-scheduler>=0.0,<0.1', 'graia-saya>=0.0,<0.1'],
 'server': ['fastapi>=0.74.1,<1.0.0', 'uvicorn[standard]>=0.17.5,<0.18.0'],
 'standard': ['arclet-alconna-graia>=0.0,<0.1',
              'graia-scheduler>=0.0,<0.1',
              'graia-saya>=0.0,<0.1',
              'prompt-toolkit>=3.0.24,<4.0.0']}

setup_kwargs = {
    'name': 'graia-ariadne',
    'version': '0.6.16',
    'description': 'Another elegant Python QQ Bot framework for mirai and mirai-api-http v2.',
    'long_description': '<div align="center">\n\n# Ariadne\n\n_Another elegant framework for mirai and mirai-api-http v2._\n\n> 接受当下, 面向未来.\n\n<a href="https://pypi.org/project/graia-ariadne"><img alt="PyPI" src="https://img.shields.io/pypi/v/graia-ariadne" /></a></td>\n<a href="https://pypi.org/project/graia-ariadne"><img alt="PyPI Pre Release" src="https://img.shields.io/github/v/tag/GraiaProject/Ariadne?include_prereleases&label=latest&color=orange"></td>\n<a href="https://pypi.org/project/graia-ariadne"><img alt="Python Version" src="https://img.shields.io/pypi/pyversions/graia-ariadne" /></a>\n<a href="https://pypi.org/project/graia-ariadne"><img alt="Python Implementation" src="https://img.shields.io/pypi/implementation/graia-ariadne" /></a>\n\n<a href="https://graia.rtfd.io/"><img alt="docs" src="https://img.shields.io/badge/文档-here-blue" /></a>\n<a href="https://graia.readthedocs.io/refs/graia/ariadne/"><img alt="API docs" src="https://img.shields.io/badge/API_文档-here-purple"></a>\n<a href="https://github.com/psf/black"><img src="https://img.shields.io/badge/code%20style-black-black.svg" alt="black" /></a>\n<a href="https://pycqa.github.io/isort/"><img src="https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat" alt="isort"/></a>\n<a href="https://github.com/GraiaProject/Ariadne/blob/master/LICENSE"><img alt="License" src="https://img.shields.io/github/license/GraiaProject/Ariadne"></a>\n\n</div>\n\n**本项目适用于 mirai-api-http 2.0 以上版本**.\n\nAriadne 是 `Graia Project` 继承了 [`Application`](https://github.com/GraiaProject/Application) 并进行了许多改进后产生的作品,\n相信它可以给你带来良好的 `Python QQ Bot` 开发体验.\n\n**注意, 本框架需要 [`mirai-api-http v2`](https://github.com/project-mirai/mirai-api-http).**\n\n## 安装\n\n`poetry add graia-ariadne`\n\n或\n\n`pip install graia-ariadne`\n\n> 我们强烈建议使用 [`poetry`](https://python-poetry.org) 进行包管理\n\n## 开始使用\n\n```python\nfrom graia.ariadne.app import Ariadne\nfrom graia.ariadne.message.chain import MessageChain\nfrom graia.ariadne.message.element import Plain\nfrom graia.ariadne.model import Friend, MiraiSession\n\napp = Ariadne(MiraiSession(host="http://localhost:8080", verify_key="ServiceVerifyKey", account=123456789))\n\n\n@app.broadcast.receiver("FriendMessage")\nasync def friend_message_listener(app: Ariadne, friend: Friend):\n    await app.sendMessage(friend, MessageChain.create([Plain("Hello, World!")]))\n\n\napp.launch_blocking()\n```\n\n更多信息请看\n[![快速开始](https://img.shields.io/badge/文档-快速开始-blue)](https://graia.readthedocs.io/quickstart/)\n\n## 讨论\n\nQQ 交流群: [邀请链接](https://jq.qq.com/?_wv=1027&k=VXp6plBD)\n\n> QQ 群不定时清除不活跃成员, 请自行重新申请入群.\n\n## 文档\n\n[![API 文档](https://img.shields.io/badge/API_文档-here-purple)](https://graia.readthedocs.io/refs/graia/ariadne/)\n[![官方文档](https://img.shields.io/badge/官方文档-here-blue)](https://graia.rtfd.io/)\n[![社区文档](https://img.shields.io/badge/社区文档-here-pink)](https://graiax.cn)\n[![鸣谢](https://img.shields.io/badge/鸣谢-here-lightgreen)](https://graia.rtfd.io/appendix/credits)\n\n**如果认为本项目有帮助, 欢迎点一个 `Star`.**\n\n## 协议\n\n本项目以 [`GNU AGPL-3.0`](https://choosealicense.com/licenses/agpl-3.0/) 作为开源协议, 这意味着你需要遵守相应的规则.\n\n## 持续集成 (CI) 状态\n\n[![Documentation Status](https://readthedocs.org/projects/graia/badge/?version=latest)](https://graia.rtfd.io/)\n[![pre-commit.ci status](https://results.pre-commit.ci/badge/github/GraiaProject/Ariadne/master.svg)](https://results.pre-commit.ci/latest/github/GraiaProject/Ariadne/master)\n\n[![Build and deploy API Docs](https://github.com/GraiaProject/Ariadne/actions/workflows/api-docs.yml/badge.svg)](https://github.com/GraiaProject/Ariadne/actions/workflows/api-docs.yml)\n[![Publish to PyPI](https://github.com/GraiaProject/Ariadne/actions/workflows/release-to-pypi.yml/badge.svg)](https://github.com/GraiaProject/Ariadne/actions/workflows/release-to-pypi.yml)\n## 开发版资源 / 参与开发\n\n[![开发分支文档](https://img.shields.io/badge/开发分支文档-here-000099)](https://graia-dev.rtfd.io/)\n[![开发分支](https://img.shields.io/badge/开发分支-here-green)](https://github.com/GraiaProject/Ariadne/tree/dev)\n[![开始开发](https://img.shields.io/badge/开始开发-here-003399)](./CONTRIBUTING.md)\n',
    'author': 'BlueGlassBlock',
    'author_email': 'blueglassblock@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://graia.rtfd.io/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
